/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "bindaudio.h"
#include "bindinterface.h"
#include "bindutil.h"

#include "beattracker.h"
#include "buffer.h"
#include "clip.h"
#include "audioport.h"
#include "mixer.h"
#include "onsetdetector.h"
#include "audioengine.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

namespace audio {

namespace binding {

// object references to types in this package
HSQOBJECT BeatTrackerObject;
HSQOBJECT BufferObject;
HSQOBJECT BufferEventObject;
HSQOBJECT ClipObject;
HSQOBJECT ConnectionObject;
HSQOBJECT ConnectorObject;
HSQOBJECT InputObject;
HSQOBJECT MixerObject;
HSQOBJECT MixerGainObject;
HSQOBJECT MultiInputObject;
HSQOBJECT MultiOutputObject;
HSQOBJECT OnsetDetectorObject;
HSQOBJECT OutputObject;
HSQOBJECT StereoInputObject;
HSQOBJECT StereoOutputObject;

//
// Audio.BeatTracker class
//
#define BIND_CLASS_NAME "BeatTracker"
#define BIND_METHOD_NAME "constructor"
SQInteger BeatTrackerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BeatTracker *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "connection" as Audio.Input
    BIND_GET_CHECK_OBJECT(connection, 1, Audio, Input, audio, AudioInputPort)
    // get parameter 2 "bpm" as float
    BIND_GET_CHECK_FLOAT(bpm, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = BeatTrackerFactory::instance().newBeatTracker(*connection, bpm))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger BeatTrackerNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.BeatTracker");
}
//
// Audio.BeatTracker start
//
#define BIND_METHOD_NAME "start"
SQInteger BeatTrackerstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.BeatTracker stop
//
#define BIND_METHOD_NAME "stop"
SQInteger BeatTrackerstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger BeatTrackerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.BeatTracker)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Buffer class
//
#define BIND_CLASS_NAME "Buffer"
SQInteger BufferCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.Buffer");
}

SQInteger BufferNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Buffer");
}
SQInteger BufferToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Buffer)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.BufferEvent class
//
#define BIND_CLASS_NAME "BufferEvent"
SQInteger BufferEventCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.BufferEvent");
}

SQInteger BufferEventNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.BufferEvent");
}
SQInteger BufferEventToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.BufferEvent)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Clip class
//
#define BIND_CLASS_NAME "Clip"
#define BIND_METHOD_NAME "constructor"
SQInteger ClipCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Clip *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "file" as string
        BIND_GET_STRING(file, 1)
        // call the implementation
        BIND_WRAP_TRY(obj = ClipFactory::instance().loadClip(file))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &bipscript::binding::bufferObject) {
        // get parameter 1 "bytes" as buffer
        BIND_GET_OBJECT(bytes, 1, , buffer, bipscript, Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj = ClipFactory::instance().loadClip(*bytes))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, buffer}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, buffer}");
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger ClipNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Clip");
}
//
// Audio.Clip buffer
//
#define BIND_METHOD_NAME "buffer"
SQInteger Clipbuffer(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clip)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "index" as integer
    BIND_GET_CHECK_INTEGER(index, 1)
    // return value
    audio::Buffer* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->buffer(index))
    // push return value
    BIND_PUSH_OBJECT(ret->addref(), Buffer, audio, Buffer, Decrement)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Audio.Clip lufs
//
#define BIND_METHOD_NAME "lufs"
SQInteger Cliplufs(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clip)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->lufs())
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Audio.Clip regain
//
#define BIND_METHOD_NAME "regain"
SQInteger Clipregain(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clip)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "factor" as float
    BIND_GET_CHECK_FLOAT(factor, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->regain(factor))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger ClipToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Clip)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Connection class
//
#define BIND_CLASS_NAME "Connection"
SQInteger ConnectionCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.Connection");
}

SQInteger ConnectionNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Connection");
}
SQInteger ConnectionToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Connection)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Connector class
//
#define BIND_CLASS_NAME "Connector"
SQInteger ConnectorCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.Connector");
}

SQInteger ConnectorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Connector");
}
//
// Audio.Connector play
//
#define BIND_METHOD_NAME "play"
SQInteger Connectorplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "clip" as Audio.Buffer
    BIND_GET_CHECK_OBJECT(clip, 1, Audio, Buffer, audio, Buffer)
    // call the implementation
    BIND_WRAP_TRY(obj->play(*clip))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Connector remove
//
#define BIND_METHOD_NAME "remove"
SQInteger Connectorremove(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "event" as Audio.BufferEvent
    BIND_GET_CHECK_OBJECT(event, 1, Audio, BufferEvent, audio, BufferEvent)
    // call the implementation
    BIND_WRAP_TRY(obj->remove(*event))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Connector schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Connectorschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "buffer" as Audio.Buffer
    BIND_GET_CHECK_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
    // get parameter 2 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 2)
    // return value
    audio::BufferEvent* ret;
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(ret = obj->schedule(*buffer, measure))
    } else {
        // get parameter 3 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
        // call the implementation
        BIND_WRAP_TRY(ret = obj->schedule(*buffer, measure, *clock))
    }
    // push return value
    BIND_PUSH_OBJECT(ret->addref(), BufferEvent, audio, BufferEvent, Decrement)
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger ConnectorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Connector)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Input class
//
#define BIND_CLASS_NAME "Input"
#define BIND_METHOD_NAME "constructor"
SQInteger InputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    AudioInputPort *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = AudioInputPortFactory::instance().newAudioInputPort(name))
    } else {
        // get parameter 2 "connection" as string
        BIND_GET_CHECK_STRING(connection, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = AudioInputPortFactory::instance().newAudioInputPort(name, connection))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger InputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Input");
}
//
// Audio.Input property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger InputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::AudioInputPort)
    // dispatch appropriate getter
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Input property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger InputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::AudioInputPort)
    // dispatch appropriate setter
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Input connect
//
#define BIND_METHOD_NAME "connect"
SQInteger Inputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(AudioInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "connection" as string
        BIND_GET_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(connection))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &bipscript::binding::regexObject) {
        // get parameter 1 "connection" as regex
        BIND_GET_OBJECT(connection, 1, , regex, bipscript, Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(*connection))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Input disconnect
//
#define BIND_METHOD_NAME "disconnect"
SQInteger Inputdisconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(AudioInputPort)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect())
    } else {
        // get parameter 1 "connection" as string
        BIND_GET_CHECK_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect(connection))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Input grab
//
#define BIND_METHOD_NAME "grab"
SQInteger Inputgrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(AudioInputPort)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger InputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Input)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Mixer class
//
#define BIND_CLASS_NAME "Mixer"
#define BIND_METHOD_NAME "constructor"
SQInteger MixerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Mixer *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "inputs" as integer
    BIND_GET_CHECK_INTEGER(inputs, 1)
    // get parameter 2 "outputs" as integer
    BIND_GET_CHECK_INTEGER(outputs, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = MixerFactory::instance().newMixer(inputs, outputs))
    } else {
        // get parameter 3 "gains" as array
        BIND_GET_CHECK_ARRAY(gains, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = MixerFactory::instance().newMixer(inputs, outputs, gains))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MixerNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Mixer");
}
//
// Audio.Mixer property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MixerGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Mixer)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MixerSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Mixer)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer add
//
#define BIND_METHOD_NAME "add"
SQInteger Mixeradd(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &ConnectionObject) {
        // get parameter 1 "connection" as Audio.Connection
        BIND_GET_OBJECT(connection, 1, Audio, Connection, audio, Connection)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*connection))
        } else {
            HSQOBJECT arg2;
            sq_getstackobj(vm, 3, &arg2);
            if(BIND_CHECK_FLOAT(2)) {
                // get parameter 2 "gain" as float
                BIND_GET_FLOAT(gain, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*connection, gain))
            }
            else if(arg2._type == OT_ARRAY) {
                BIND_NARGS_CHECK_MIN(2) // check min parameter count
                // get parameter 2 "gain" as array
                BIND_GET_ARRAY(gain, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*connection, gain))
            }
            else {
                return sq_throwerror(vm, "argument 2 is not of type {float, array}");
            }
        }
    }
    else if(audio::Source *source = audio::binding::getSource(arg1type, &arg1)) {
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*source))
        } else {
            HSQOBJECT arg2;
            sq_getstackobj(vm, 3, &arg2);
            if(BIND_CHECK_FLOAT(2)) {
                // get parameter 2 "gain" as float
                BIND_GET_FLOAT(gain, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*source, gain))
            }
            else if(arg2._type == OT_ARRAY) {
                BIND_NARGS_CHECK_MIN(2) // check min parameter count
                // get parameter 2 "gains" as array
                BIND_GET_ARRAY(gains, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*source, gains))
            }
            else {
                return sq_throwerror(vm, "argument 2 is not of type {float, array}");
            }
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Connection, Audio.Source}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Connection, Audio.Source}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer gain
//
#define BIND_METHOD_NAME "gain"
SQInteger Mixergain(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    // get parameter 1 "input" as integer
    BIND_GET_CHECK_INTEGER(input, 1)
    HSQOBJECT arg2;
    sq_getstackobj(vm, 3, &arg2);
    if(arg2._type == OT_INTEGER) {
        BIND_NARGS_CHECK_MIN(3) // check min parameter count
        BIND_NARGS_CHECK_MAX(3) // check max parameter count
        // get parameter 2 "output" as integer
        BIND_GET_INTEGER(output, 2)
        // get parameter 3 "gain" as float
        BIND_GET_CHECK_FLOAT(gain, 3)
        // call the implementation
        BIND_WRAP_TRY(obj->gain(input, output, gain))
    }
    else if(arg2._type == OT_ARRAY) {
        BIND_NARGS_CHECK_MAX(2) // check max parameter count
        // get parameter 2 "gains" as array
        BIND_GET_ARRAY(gains, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->gain(input, gains))
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {integer, array}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer grab
//
#define BIND_METHOD_NAME "grab"
SQInteger Mixergrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer play
//
#define BIND_METHOD_NAME "play"
SQInteger Mixerplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Mixerschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &MixerGainObject) {
        // get parameter 1 "gain" as Audio.MixerGain
        BIND_GET_OBJECT(gain, 1, Audio, MixerGain, audio, MixerGain)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*gain, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*gain, measure, *clock))
        }
    }
    else if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.MixerGain, Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.MixerGain, Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MixerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Mixer)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.MixerGain class
//
#define BIND_CLASS_NAME "MixerGain"
#define BIND_METHOD_NAME "constructor"
SQInteger MixerGainCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MixerGain *obj;
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "input" as integer
    BIND_GET_CHECK_INTEGER(input, 1)
    // get parameter 2 "output" as integer
    BIND_GET_CHECK_INTEGER(output, 2)
    // get parameter 3 "amount" as float
    BIND_GET_CHECK_FLOAT(amount, 3)
    // call the implementation
    BIND_WRAP_TRY(obj = new MixerGain(input, output, amount))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<MixerGain>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Audio.MixerGain property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MixerGainGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MixerGain)
    // dispatch appropriate getter
    if(!strcmp(key, "amount")) {
        SQFloat ret = obj->amount();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MixerGain property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MixerGainSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MixerGain)
    // dispatch appropriate setter
    if(!strcmp(key, "amount")) {
        return sq_throwerror(vm, "property \"amount\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MixerGainToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.MixerGain)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.MultiInput class
//
#define BIND_CLASS_NAME "MultiInput"
#define BIND_METHOD_NAME "constructor"
SQInteger MultiInputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MultiInput *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // get parameter 2 "channels" as integer
    BIND_GET_CHECK_INTEGER(channels, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newInputPortSet(name, channels))
    } else {
        // get parameter 3 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newInputPortSet(name, channels, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MultiInputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.MultiInput");
}
//
// Audio.MultiInput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MultiInputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiInput)
    // dispatch appropriate getter
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiInput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MultiInputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiInput)
    // dispatch appropriate setter
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiInput connect
//
#define BIND_METHOD_NAME "connect"
SQInteger MultiInputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiInput)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "index" as integer
    BIND_GET_CHECK_INTEGER(index, 1)
    HSQOBJECT arg2;
    sq_getstackobj(vm, 3, &arg2);
    if(arg2._type == OT_STRING) {
        // get parameter 2 "name" as string
        BIND_GET_STRING(name, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, name))
    }
    else if(arg2._type == OT_INSTANCE) {
    SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
    if(arg2type == &bipscript::binding::regexObject) {
        // get parameter 2 "name" as regex
        BIND_GET_OBJECT(name, 2, , regex, bipscript, Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, *name))
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.MultiInput grab
//
#define BIND_METHOD_NAME "grab"
SQInteger MultiInputgrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiInput)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MultiInputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.MultiInput)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.MultiOutput class
//
#define BIND_CLASS_NAME "MultiOutput"
#define BIND_METHOD_NAME "constructor"
SQInteger MultiOutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MultiOutput *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // get parameter 2 "channels" as integer
    BIND_GET_CHECK_INTEGER(channels, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newOutputPortSet(name, channels))
    } else {
        // get parameter 3 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newOutputPortSet(name, channels, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MultiOutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.MultiOutput");
}
//
// Audio.MultiOutput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MultiOutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiOutput)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiOutput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MultiOutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiOutput)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiOutput play
//
#define BIND_METHOD_NAME "play"
SQInteger MultiOutputplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiOutput)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.MultiOutput schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger MultiOutputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiOutput)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MultiOutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.MultiOutput)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.OnsetDetector class
//
#define BIND_CLASS_NAME "OnsetDetector"
#define BIND_METHOD_NAME "constructor"
SQInteger OnsetDetectorCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    OnsetDetector *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = OnsetDetectorFactory::instance().newOnsetDetector())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger OnsetDetectorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.OnsetDetector");
}
//
// Audio.OnsetDetector property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger OnsetDetectorGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::OnsetDetector)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "threshold")) {
        SQFloat ret = obj->threshold();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger OnsetDetectorSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::OnsetDetector)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    if(!strcmp(key, "threshold")) {
        // get parameter 2 "threshold" as float
        BIND_GET_CHECK_FLOAT(threshold, 2)
        BIND_WRAP_TRY(obj->threshold(threshold))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector onOnset
//
#define BIND_METHOD_NAME "onOnset"
SQInteger OnsetDetectoronOnset(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(OnsetDetector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onOnset(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector play
//
#define BIND_METHOD_NAME "play"
SQInteger OnsetDetectorplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(OnsetDetector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger OnsetDetectorschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(OnsetDetector)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger OnsetDetectorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.OnsetDetector)", 21);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Output class
//
#define BIND_CLASS_NAME "Output"
#define BIND_METHOD_NAME "constructor"
SQInteger OutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Output *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newAudioOutputPort(name))
    } else {
        HSQOBJECT arg2;
        sq_getstackobj(vm, 3, &arg2);
        if(arg2._type == OT_STRING) {
            // get parameter 2 "connection" as string
            BIND_GET_STRING(connection, 2)
            // call the implementation
            BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newAudioOutputPort(name, connection))
        }
        else if(arg2._type == OT_BOOL) {
            BIND_NARGS_CHECK_MIN(2) // check min parameter count
            // get parameter 2 "autoconnect" as bool
            BIND_GET_BOOL(autoconnect, 2)
            // call the implementation
            BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newAudioOutputPort(name, autoconnect))
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {string, bool}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger OutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Output");
}
//
// Audio.Output property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger OutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Output)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Output property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger OutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Output)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Output connect
//
#define BIND_METHOD_NAME "connect"
SQInteger Outputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "connection" as string
        BIND_GET_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(connection))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &bipscript::binding::regexObject) {
        // get parameter 1 "connection" as regex
        BIND_GET_OBJECT(connection, 1, , regex, bipscript, Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(*connection))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Output disconnect
//
#define BIND_METHOD_NAME "disconnect"
SQInteger Outputdisconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect())
    } else {
        // get parameter 1 "connection" as string
        BIND_GET_CHECK_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect(connection))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Output play
//
#define BIND_METHOD_NAME "play"
SQInteger Outputplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Output schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Outputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger OutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Output)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.StereoInput class
//
#define BIND_CLASS_NAME "StereoInput"
#define BIND_METHOD_NAME "constructor"
SQInteger StereoInputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    StereoInput *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newStereoInputPort(name))
    } else {
        // get parameter 2 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newStereoInputPort(name, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger StereoInputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.StereoInput");
}
//
// Audio.StereoInput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger StereoInputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoInput)
    // dispatch appropriate getter
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoInput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger StereoInputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoInput)
    // dispatch appropriate setter
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoInput connect
//
#define BIND_METHOD_NAME "connect"
SQInteger StereoInputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoInput)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "index" as integer
    BIND_GET_CHECK_INTEGER(index, 1)
    HSQOBJECT arg2;
    sq_getstackobj(vm, 3, &arg2);
    if(arg2._type == OT_STRING) {
        // get parameter 2 "name" as string
        BIND_GET_STRING(name, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, name))
    }
    else if(arg2._type == OT_INSTANCE) {
    SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
    if(arg2type == &bipscript::binding::regexObject) {
        // get parameter 2 "name" as regex
        BIND_GET_OBJECT(name, 2, , regex, bipscript, Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, *name))
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.StereoInput grab
//
#define BIND_METHOD_NAME "grab"
SQInteger StereoInputgrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoInput)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger StereoInputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.StereoInput)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.StereoOutput class
//
#define BIND_CLASS_NAME "StereoOutput"
#define BIND_METHOD_NAME "constructor"
SQInteger StereoOutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    StereoOutput *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newStereoOutputPort(name))
    } else {
        // get parameter 2 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newStereoOutputPort(name, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger StereoOutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.StereoOutput");
}
//
// Audio.StereoOutput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger StereoOutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoOutput)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoOutput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger StereoOutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoOutput)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoOutput play
//
#define BIND_METHOD_NAME "play"
SQInteger StereoOutputplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoOutput)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.StereoOutput schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger StereoOutputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoOutput)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger StereoOutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.StereoOutput)", 20);
    return 1;
}
#undef BIND_CLASS_NAME

SQInteger AudioNewSlot(HSQUIRRELVM vm)
{
	return sq_throwerror(vm, "package table does not support the new slot operator");
}

void bindAudio(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Audio", 5);

    // create class Audio.BeatTracker
    binding.startClass("BeatTracker", 11, BeatTrackerObject);
    binding.addMethod("constructor", 11, &BeatTrackerCtor); // ctor
    binding.addMethod("_cloned", 7, &BeatTrackerNoclone); // unclonable
    binding.addMethod("start", 5, &BeatTrackerstart);
    binding.addMethod("stop", 4, &BeatTrackerstop);
    binding.addMethod("_tostring", 9, &BeatTrackerToString);
    binding.endClass();

    // create class Audio.Buffer
    binding.startClass("Buffer", 6, BufferObject);
    binding.addMethod("constructor", 11, &BufferCtor); // ctor
    binding.addMethod("_cloned", 7, &BufferNoclone); // unclonable
    binding.addMethod("_tostring", 9, &BufferToString);
    binding.endClass();

    // create class Audio.BufferEvent
    binding.startClass("BufferEvent", 11, BufferEventObject);
    binding.addMethod("constructor", 11, &BufferEventCtor); // ctor
    binding.addMethod("_cloned", 7, &BufferEventNoclone); // unclonable
    binding.addMethod("_tostring", 9, &BufferEventToString);
    binding.endClass();

    // create class Audio.Clip
    binding.startClass("Clip", 4, ClipObject);
    binding.addMethod("constructor", 11, &ClipCtor); // ctor
    binding.addMethod("_cloned", 7, &ClipNoclone); // unclonable
    binding.addMethod("buffer", 6, &Clipbuffer);
    binding.addMethod("lufs", 4, &Cliplufs);
    binding.addMethod("regain", 6, &Clipregain);
    binding.addMethod("_tostring", 9, &ClipToString);
    binding.endClass();

    // create class Audio.Connection
    binding.startClass("Connection", 10, ConnectionObject);
    binding.addMethod("constructor", 11, &ConnectionCtor); // ctor
    binding.addMethod("_cloned", 7, &ConnectionNoclone); // unclonable
    binding.addMethod("_tostring", 9, &ConnectionToString);
    binding.endClass();

    // create class Audio.Connector
    binding.startClass("Connector", 9, ConnectorObject);
    binding.addMethod("constructor", 11, &ConnectorCtor); // ctor
    binding.addMethod("_cloned", 7, &ConnectorNoclone); // unclonable
    binding.addMethod("play", 4, &Connectorplay);
    binding.addMethod("remove", 6, &Connectorremove);
    binding.addMethod("schedule", 8, &Connectorschedule);
    binding.addMethod("_tostring", 9, &ConnectorToString);
    binding.endClass();

    // create class Audio.Input
    binding.startClass("Input", 5, InputObject);
    binding.addMethod("constructor", 11, &InputCtor); // ctor
    binding.addMethod("_cloned", 7, &InputNoclone); // unclonable
    binding.addMethod("_get", 4, &InputGetters);
    binding.addMethod("_set", 4, &InputSetters);
    binding.addMethod("connect", 7, &Inputconnect);
    binding.addMethod("disconnect", 10, &Inputdisconnect);
    binding.addMethod("grab", 4, &Inputgrab);
    binding.addMethod("_tostring", 9, &InputToString);
    binding.endClass();

    // create class Audio.Mixer
    binding.startClass("Mixer", 5, MixerObject);
    binding.addMethod("constructor", 11, &MixerCtor); // ctor
    binding.addMethod("_cloned", 7, &MixerNoclone); // unclonable
    binding.addMethod("_get", 4, &MixerGetters);
    binding.addMethod("_set", 4, &MixerSetters);
    binding.addMethod("add", 3, &Mixeradd);
    binding.addMethod("gain", 4, &Mixergain);
    binding.addMethod("grab", 4, &Mixergrab);
    binding.addMethod("play", 4, &Mixerplay);
    binding.addMethod("schedule", 8, &Mixerschedule);
    binding.addMethod("_tostring", 9, &MixerToString);
    binding.endClass();

    // create class Audio.MixerGain
    binding.startClass("MixerGain", 9, MixerGainObject);
    binding.addMethod("constructor", 11, &MixerGainCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<MixerGain>); // clone
    binding.addMethod("_get", 4, &MixerGainGetters);
    binding.addMethod("_set", 4, &MixerGainSetters);
    binding.addMethod("_tostring", 9, &MixerGainToString);
    binding.endClass();

    // create class Audio.MultiInput
    binding.startClass("MultiInput", 10, MultiInputObject);
    binding.addMethod("constructor", 11, &MultiInputCtor); // ctor
    binding.addMethod("_cloned", 7, &MultiInputNoclone); // unclonable
    binding.addMethod("_get", 4, &MultiInputGetters);
    binding.addMethod("_set", 4, &MultiInputSetters);
    binding.addMethod("connect", 7, &MultiInputconnect);
    binding.addMethod("grab", 4, &MultiInputgrab);
    binding.addMethod("_tostring", 9, &MultiInputToString);
    binding.endClass();

    // create class Audio.MultiOutput
    binding.startClass("MultiOutput", 11, MultiOutputObject);
    binding.addMethod("constructor", 11, &MultiOutputCtor); // ctor
    binding.addMethod("_cloned", 7, &MultiOutputNoclone); // unclonable
    binding.addMethod("_get", 4, &MultiOutputGetters);
    binding.addMethod("_set", 4, &MultiOutputSetters);
    binding.addMethod("play", 4, &MultiOutputplay);
    binding.addMethod("schedule", 8, &MultiOutputschedule);
    binding.addMethod("_tostring", 9, &MultiOutputToString);
    binding.endClass();

    // create class Audio.OnsetDetector
    binding.startClass("OnsetDetector", 13, OnsetDetectorObject);
    binding.addMethod("constructor", 11, &OnsetDetectorCtor); // ctor
    binding.addMethod("_cloned", 7, &OnsetDetectorNoclone); // unclonable
    binding.addMethod("_get", 4, &OnsetDetectorGetters);
    binding.addMethod("_set", 4, &OnsetDetectorSetters);
    binding.addMethod("onOnset", 7, &OnsetDetectoronOnset);
    binding.addMethod("play", 4, &OnsetDetectorplay);
    binding.addMethod("schedule", 8, &OnsetDetectorschedule);
    binding.addMethod("_tostring", 9, &OnsetDetectorToString);
    binding.endClass();

    // create class Audio.Output
    binding.startClass("Output", 6, OutputObject);
    binding.addMethod("constructor", 11, &OutputCtor); // ctor
    binding.addMethod("_cloned", 7, &OutputNoclone); // unclonable
    binding.addMethod("_get", 4, &OutputGetters);
    binding.addMethod("_set", 4, &OutputSetters);
    binding.addMethod("connect", 7, &Outputconnect);
    binding.addMethod("disconnect", 10, &Outputdisconnect);
    binding.addMethod("play", 4, &Outputplay);
    binding.addMethod("schedule", 8, &Outputschedule);
    binding.addMethod("_tostring", 9, &OutputToString);
    binding.endClass();

    // create class Audio.StereoInput
    binding.startClass("StereoInput", 11, StereoInputObject);
    binding.addMethod("constructor", 11, &StereoInputCtor); // ctor
    binding.addMethod("_cloned", 7, &StereoInputNoclone); // unclonable
    binding.addMethod("_get", 4, &StereoInputGetters);
    binding.addMethod("_set", 4, &StereoInputSetters);
    binding.addMethod("connect", 7, &StereoInputconnect);
    binding.addMethod("grab", 4, &StereoInputgrab);
    binding.addMethod("_tostring", 9, &StereoInputToString);
    binding.endClass();

    // create class Audio.StereoOutput
    binding.startClass("StereoOutput", 12, StereoOutputObject);
    binding.addMethod("constructor", 11, &StereoOutputCtor); // ctor
    binding.addMethod("_cloned", 7, &StereoOutputNoclone); // unclonable
    binding.addMethod("_get", 4, &StereoOutputGetters);
    binding.addMethod("_set", 4, &StereoOutputSetters);
    binding.addMethod("play", 4, &StereoOutputplay);
    binding.addMethod("schedule", 8, &StereoOutputschedule);
    binding.addMethod("_tostring", 9, &StereoOutputToString);
    binding.endClass();

    sq_newtable(vm); // delegate table for package properties
    binding.addMethod("_newslot", 8, &AudioNewSlot);
    sq_setdelegate(vm, -2);

    binding.endTable(); // end package table
}
}}}
