/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "bindmath.h"
#include "bindinterface.h"
#include "bindutil.h"

#include "random.h"
#include "mathfunc.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

namespace math {

namespace binding {

// object references to types in this package
HSQOBJECT RandomObject;

#define BIND_CLASS_NAME "Math"
//
// Math abs
//
#define BIND_METHOD_NAME "abs"
SQInteger Mathabs(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INTEGER) {
        // get parameter 1 "value" as integer
        BIND_GET_INTEGER(value, 1)
        // return value
        SQInteger ret;
        // call the implementation
        BIND_WRAP_TRY(ret = Math::abs(value))
        // push return value
        BIND_PUSH_INTEGER(ret)
        return 1;

    }
    else if(BIND_CHECK_FLOAT(1)) {
        // get parameter 1 "value" as float
        BIND_GET_FLOAT(value, 1)
        // return value
        SQFloat ret;
        // call the implementation
        BIND_WRAP_TRY(ret = Math::abs(value))
        // push return value
        BIND_PUSH_FLOAT(ret)
        return 1;

    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {integer, float}");
    }}
#undef BIND_METHOD_NAME

//
// Math acos
//
#define BIND_METHOD_NAME "acos"
SQInteger Mathacos(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::acos(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math asin
//
#define BIND_METHOD_NAME "asin"
SQInteger Mathasin(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::asin(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math atan
//
#define BIND_METHOD_NAME "atan"
SQInteger Mathatan(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::atan(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math atan2
//
#define BIND_METHOD_NAME "atan2"
SQInteger Mathatan2(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "x" as float
    BIND_GET_CHECK_FLOAT(x, 1)
    // get parameter 2 "y" as float
    BIND_GET_CHECK_FLOAT(y, 2)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::atan2(x, y))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math cbrt
//
#define BIND_METHOD_NAME "cbrt"
SQInteger Mathcbrt(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "cubed" as float
    BIND_GET_CHECK_FLOAT(cubed, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::cbrt(cubed))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math ceil
//
#define BIND_METHOD_NAME "ceil"
SQInteger Mathceil(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::ceil(value))
    // push return value
    BIND_PUSH_INTEGER(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math cos
//
#define BIND_METHOD_NAME "cos"
SQInteger Mathcos(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::cos(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math exp
//
#define BIND_METHOD_NAME "exp"
SQInteger Mathexp(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::exp(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math floor
//
#define BIND_METHOD_NAME "floor"
SQInteger Mathfloor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::floor(value))
    // push return value
    BIND_PUSH_INTEGER(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math log
//
#define BIND_METHOD_NAME "log"
SQInteger Mathlog(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::log(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math log10
//
#define BIND_METHOD_NAME "log10"
SQInteger Mathlog10(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::log10(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math max
//
#define BIND_METHOD_NAME "max"
SQInteger Mathmax(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "x" as float
    BIND_GET_CHECK_FLOAT(x, 1)
    // get parameter 2 "y" as float
    BIND_GET_CHECK_FLOAT(y, 2)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::max(x, y))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math min
//
#define BIND_METHOD_NAME "min"
SQInteger Mathmin(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "x" as float
    BIND_GET_CHECK_FLOAT(x, 1)
    // get parameter 2 "y" as float
    BIND_GET_CHECK_FLOAT(y, 2)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::min(x, y))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math pow
//
#define BIND_METHOD_NAME "pow"
SQInteger Mathpow(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "base" as float
    BIND_GET_CHECK_FLOAT(base, 1)
    // get parameter 2 "exponent" as float
    BIND_GET_CHECK_FLOAT(exponent, 2)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::pow(base, exponent))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math round
//
#define BIND_METHOD_NAME "round"
SQInteger Mathround(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::round(value))
    // push return value
    BIND_PUSH_INTEGER(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math sin
//
#define BIND_METHOD_NAME "sin"
SQInteger Mathsin(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::sin(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math sqrt
//
#define BIND_METHOD_NAME "sqrt"
SQInteger Mathsqrt(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "squared" as float
    BIND_GET_CHECK_FLOAT(squared, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::sqrt(squared))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math tan
//
#define BIND_METHOD_NAME "tan"
SQInteger Mathtan(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "value" as float
    BIND_GET_CHECK_FLOAT(value, 1)
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Math::tan(value))
    // push return value
    BIND_PUSH_FLOAT(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math package property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MathGetters(HSQUIRRELVM vm)
{
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    if(!strcmp(key, "E")) {
        SQFloat ret = math::Math::E();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    if(!strcmp(key, "PI")) {
        SQFloat ret = math::Math::PI();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Math property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MathSetters(HSQUIRRELVM vm)
{
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    if(!strcmp(key, "E")) {
        return sq_throwerror(vm, "property \"E\" is read-only");
    }
    if(!strcmp(key, "PI")) {
        return sq_throwerror(vm, "property \"PI\" is read-only");
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

#undef BIND_CLASS_NAME
//
// Math.Random class
//
#define BIND_CLASS_NAME "Random"
#define BIND_METHOD_NAME "constructor"
SQInteger RandomCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Random *obj;
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj = new Random())
    } else {
        BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
        // get parameter 1 "seed" as integer
        BIND_GET_CHECK_INTEGER(seed, 1)
        // call the implementation
        BIND_WRAP_TRY(obj = new Random(seed))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Random>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Math.Random integer
//
#define BIND_METHOD_NAME "integer"
SQInteger Randominteger(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Random)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "max" as integer
    BIND_GET_CHECK_INTEGER(max, 1)
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->integer(max))
    // push return value
    BIND_PUSH_INTEGER(ret)
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger RandomToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Math.Random)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

SQInteger MathNewSlot(HSQUIRRELVM vm)
{
	return sq_throwerror(vm, "package table does not support the new slot operator");
}

void bindMath(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Math", 4);

    // static methods
    binding.addMethod("abs", 3, &Mathabs);
    binding.addMethod("abs", 3, &Mathabs);
    binding.addMethod("acos", 4, &Mathacos);
    binding.addMethod("asin", 4, &Mathasin);
    binding.addMethod("atan", 4, &Mathatan);
    binding.addMethod("atan2", 5, &Mathatan2);
    binding.addMethod("cbrt", 4, &Mathcbrt);
    binding.addMethod("ceil", 4, &Mathceil);
    binding.addMethod("cos", 3, &Mathcos);
    binding.addMethod("exp", 3, &Mathexp);
    binding.addMethod("floor", 5, &Mathfloor);
    binding.addMethod("log", 3, &Mathlog);
    binding.addMethod("log10", 5, &Mathlog10);
    binding.addMethod("max", 3, &Mathmax);
    binding.addMethod("min", 3, &Mathmin);
    binding.addMethod("pow", 3, &Mathpow);
    binding.addMethod("round", 5, &Mathround);
    binding.addMethod("sin", 3, &Mathsin);
    binding.addMethod("sqrt", 4, &Mathsqrt);
    binding.addMethod("tan", 3, &Mathtan);

    // create class Math.Random
    binding.startClass("Random", 6, RandomObject);
    binding.addMethod("constructor", 11, &RandomCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Random>); // clone
    binding.addMethod("integer", 7, &Randominteger);
    binding.addMethod("_tostring", 9, &RandomToString);
    binding.endClass();

    sq_newtable(vm); // delegate table for package properties
    binding.addMethod("_get", 4, &MathGetters);
    binding.addMethod("_set", 4, &MathSetters);
    binding.addMethod("_newslot", 8, &MathNewSlot);
    sq_setdelegate(vm, -2);

    binding.endTable(); // end package table
}
}}}
