/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "bindtime.h"
#include "bindinterface.h"
#include "bindutil.h"

#include "basicclock.h"
#include "link.h"
#include "variableclock.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

namespace time {

namespace binding {

// object references to types in this package
HSQOBJECT BasicClockObject;
HSQOBJECT LinkObject;
HSQOBJECT SignatureObject;
HSQOBJECT VariableClockObject;

#define BIND_CLASS_NAME "Time"
//
// Time package property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger TimeGetters(HSQUIRRELVM vm)
{
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    if(!strcmp(key, "def")) {
        time::Clock* ret = time::Time::def();
        time::binding::pushClock(vm, ret);
        return 1;
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger TimeSetters(HSQUIRRELVM vm)
{
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    if(!strcmp(key, "def")) {
        // get parameter 2 "def" as Time.Clock
        BIND_GET_CHECK_INTERFACE(def, 2, Time, Clock, time, Clock)
        BIND_WRAP_TRY(time::Time::def(def))
        return 0;
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

#undef BIND_CLASS_NAME
//
// Time.BasicClock class
//
#define BIND_CLASS_NAME "BasicClock"
#define BIND_METHOD_NAME "constructor"
SQInteger BasicClockCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BasicClock *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "bpm" as float
    BIND_GET_CHECK_FLOAT(bpm, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = BasicClockFactory::instance().basicClock(bpm))
    } else {
        // get parameter 2 "time" as Time.Signature
        BIND_GET_CHECK_OBJECT(time, 2, Time, Signature, time, Signature)
        // call the implementation
        BIND_WRAP_TRY(obj = BasicClockFactory::instance().basicClock(bpm, *time))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger BasicClockNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Time.BasicClock");
}
//
// Time.BasicClock property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger BasicClockGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::BasicClock)
    // dispatch appropriate getter
    if(!strcmp(key, "signature")) {
        time::Signature* ret = obj->signature();
        BIND_PUSH_OBJECT(ret->addref(), Signature, time, Signature, Decrement)
        return 1;
    }
    if(!strcmp(key, "tempo")) {
        SQFloat ret = obj->tempo();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time.BasicClock property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger BasicClockSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::BasicClock)
    // dispatch appropriate setter
    if(!strcmp(key, "signature")) {
        // get parameter 2 "signature" as Time.Signature
        BIND_GET_CHECK_OBJECT(signature, 2, Time, Signature, time, Signature)
        BIND_WRAP_TRY(obj->signature(signature))
        return 0;
    }
    if(!strcmp(key, "tempo")) {
        // get parameter 2 "tempo" as float
        BIND_GET_CHECK_FLOAT(tempo, 2)
        BIND_WRAP_TRY(obj->tempo(tempo))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time.BasicClock restart
//
#define BIND_METHOD_NAME "restart"
SQInteger BasicClockrestart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BasicClock)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->restart())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Time.BasicClock start
//
#define BIND_METHOD_NAME "start"
SQInteger BasicClockstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BasicClock)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Time.BasicClock stop
//
#define BIND_METHOD_NAME "stop"
SQInteger BasicClockstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BasicClock)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger BasicClockToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Time.BasicClock)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Time.Link class
//
#define BIND_CLASS_NAME "Link"
#define BIND_METHOD_NAME "constructor"
SQInteger LinkCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Link *obj;
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj = LinkFactory::instance().newLink())
    } else {
        // get parameter 1 "bpm" as float
        BIND_GET_CHECK_FLOAT(bpm, 1)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj = LinkFactory::instance().newLink(bpm))
        } else {
            // get parameter 2 "time" as Time.Signature
            BIND_GET_CHECK_OBJECT(time, 2, Time, Signature, time, Signature)
            // call the implementation
            BIND_WRAP_TRY(obj = LinkFactory::instance().newLink(bpm, *time))
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger LinkNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Time.Link");
}
//
// Time.Link property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger LinkGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::Link)
    // dispatch appropriate getter
    if(!strcmp(key, "tempo")) {
        SQFloat ret = obj->tempo();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time.Link property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger LinkSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::Link)
    // dispatch appropriate setter
    if(!strcmp(key, "tempo")) {
        // get parameter 2 "tempo" as float
        BIND_GET_CHECK_FLOAT(tempo, 2)
        BIND_WRAP_TRY(obj->tempo(tempo))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time.Link schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Linkschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Link)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "tempo" as float
    BIND_GET_CHECK_FLOAT(tempo, 1)
    // get parameter 2 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 2)
    // call the implementation
    BIND_WRAP_TRY(obj->schedule(tempo, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Time.Link start
//
#define BIND_METHOD_NAME "start"
SQInteger Linkstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Link)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Time.Link stop
//
#define BIND_METHOD_NAME "stop"
SQInteger Linkstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Link)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger LinkToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Time.Link)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Time.Signature class
//
#define BIND_CLASS_NAME "Signature"
#define BIND_METHOD_NAME "constructor"
SQInteger SignatureCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Signature *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "numerator" as integer
    BIND_GET_CHECK_INTEGER(numerator, 1)
    // get parameter 2 "denominator" as integer
    BIND_GET_CHECK_INTEGER(denominator, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = new Signature(numerator, denominator))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Signature>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Time.Signature property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger SignatureGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::Signature)
    // dispatch appropriate getter
    if(!strcmp(key, "denominator")) {
        SQInteger ret = obj->denominator();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "numerator")) {
        SQInteger ret = obj->numerator();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time.Signature property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger SignatureSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::Signature)
    // dispatch appropriate setter
    if(!strcmp(key, "denominator")) {
        // get parameter 2 "denominator" as integer
        BIND_GET_CHECK_INTEGER(denominator, 2)
        BIND_WRAP_TRY(obj->denominator(denominator))
        return 0;
    }
    if(!strcmp(key, "numerator")) {
        // get parameter 2 "numerator" as integer
        BIND_GET_CHECK_INTEGER(numerator, 2)
        BIND_WRAP_TRY(obj->numerator(numerator))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger SignatureToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Time.Signature)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Time.VariableClock class
//
#define BIND_CLASS_NAME "VariableClock"
#define BIND_METHOD_NAME "constructor"
SQInteger VariableClockCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    VariableClock *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "bpm" as float
    BIND_GET_CHECK_FLOAT(bpm, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = VariableClockFactory::instance().variableClock(bpm))
    } else {
        // get parameter 2 "time" as Time.Signature
        BIND_GET_CHECK_OBJECT(time, 2, Time, Signature, time, Signature)
        // call the implementation
        BIND_WRAP_TRY(obj = VariableClockFactory::instance().variableClock(bpm, *time))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger VariableClockNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Time.VariableClock");
}
//
// Time.VariableClock property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger VariableClockGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::VariableClock)
    // dispatch appropriate getter
    if(!strcmp(key, "tempo")) {
        SQFloat ret = obj->tempo();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time.VariableClock property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger VariableClockSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(time::VariableClock)
    // dispatch appropriate setter
    if(!strcmp(key, "tempo")) {
        // get parameter 2 "tempo" as float
        BIND_GET_CHECK_FLOAT(tempo, 2)
        BIND_WRAP_TRY(obj->tempo(tempo))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Time.VariableClock reposition
//
#define BIND_METHOD_NAME "reposition"
SQInteger VariableClockreposition(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(VariableClock)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "location" as float
    BIND_GET_CHECK_FLOAT(location, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj->reposition(location))
    } else {
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->reposition(location, measure))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Time.VariableClock schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger VariableClockschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(VariableClock)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(BIND_CHECK_FLOAT(1)) {
        // get parameter 1 "tempo" as float
        BIND_GET_FLOAT(tempo, 1)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->schedule(tempo, measure))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &SignatureObject) {
        // get parameter 1 "signature" as Time.Signature
        BIND_GET_OBJECT(signature, 1, Time, Signature, time, Signature)
        // get parameter 2 "bar" as integer
        BIND_GET_CHECK_INTEGER(bar, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->schedule(*signature, bar))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Time.Signature, float}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Time.Signature, float}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Time.VariableClock start
//
#define BIND_METHOD_NAME "start"
SQInteger VariableClockstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(VariableClock)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Time.VariableClock stop
//
#define BIND_METHOD_NAME "stop"
SQInteger VariableClockstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(VariableClock)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger VariableClockToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Time.VariableClock)", 20);
    return 1;
}
#undef BIND_CLASS_NAME

SQInteger TimeNewSlot(HSQUIRRELVM vm)
{
	return sq_throwerror(vm, "package table does not support the new slot operator");
}

void bindTime(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Time", 4);

    // create class Time.BasicClock
    binding.startClass("BasicClock", 10, BasicClockObject);
    binding.addMethod("constructor", 11, &BasicClockCtor); // ctor
    binding.addMethod("_cloned", 7, &BasicClockNoclone); // unclonable
    binding.addMethod("_get", 4, &BasicClockGetters);
    binding.addMethod("_set", 4, &BasicClockSetters);
    binding.addMethod("restart", 7, &BasicClockrestart);
    binding.addMethod("start", 5, &BasicClockstart);
    binding.addMethod("stop", 4, &BasicClockstop);
    binding.addMethod("_tostring", 9, &BasicClockToString);
    binding.endClass();

    // create class Time.Link
    binding.startClass("Link", 4, LinkObject);
    binding.addMethod("constructor", 11, &LinkCtor); // ctor
    binding.addMethod("_cloned", 7, &LinkNoclone); // unclonable
    binding.addMethod("_get", 4, &LinkGetters);
    binding.addMethod("_set", 4, &LinkSetters);
    binding.addMethod("schedule", 8, &Linkschedule);
    binding.addMethod("start", 5, &Linkstart);
    binding.addMethod("stop", 4, &Linkstop);
    binding.addMethod("_tostring", 9, &LinkToString);
    binding.endClass();

    // create class Time.Signature
    binding.startClass("Signature", 9, SignatureObject);
    binding.addMethod("constructor", 11, &SignatureCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Signature>); // clone
    binding.addMethod("_get", 4, &SignatureGetters);
    binding.addMethod("_set", 4, &SignatureSetters);
    binding.addMethod("_tostring", 9, &SignatureToString);
    binding.endClass();

    // create class Time.VariableClock
    binding.startClass("VariableClock", 13, VariableClockObject);
    binding.addMethod("constructor", 11, &VariableClockCtor); // ctor
    binding.addMethod("_cloned", 7, &VariableClockNoclone); // unclonable
    binding.addMethod("_get", 4, &VariableClockGetters);
    binding.addMethod("_set", 4, &VariableClockSetters);
    binding.addMethod("reposition", 10, &VariableClockreposition);
    binding.addMethod("schedule", 8, &VariableClockschedule);
    binding.addMethod("start", 5, &VariableClockstart);
    binding.addMethod("stop", 4, &VariableClockstop);
    binding.addMethod("_tostring", 9, &VariableClockToString);
    binding.endClass();

    sq_newtable(vm); // delegate table for package properties
    binding.addMethod("_get", 4, &TimeGetters);
    binding.addMethod("_set", 4, &TimeSetters);
    binding.addMethod("_newslot", 8, &TimeNewSlot);
    sq_setdelegate(vm, -2);

    binding.endTable(); // end package table
}
}}}
