/*  YRX - re_.h
**  (C) 2005 by Remo Dentato
**
** Permission to use, copy, modify and distribute this code and
** its documentation for any purpose is hereby granted without fee,
** provided that the above copyright notice appear in all copies
** and supporting documentation.
** 
** Copyright holder makes no representations about the suitability
** of this software for any purpose. It is provided "as is" without
** express or implied warranty.
**
** This code is a derivative work of the PUBLIC DOMAIN regex routines
** by Ozan Yigit: http://www.cs.yorku.ca/~oz/
**
*/

#ifndef _RE0_H_
#define _RE0_H_

#define MAXTAG  10

#define OKP     1
#define NOP     0

/*
 *   Bit pattern is meaningful!
 *   Do not change unless you know what you're doing!
 */

#define QSTR    0x21  /* 0010 0001 */
#define GOAL    0x22  /* 0010 0010 */
#define NINT    0x23  /* 0010 0011 */
#define NFLOAT  0x24  /* 0010 0100 */
#define NHEX    0x25  /* 0010 0101 */

#define RCALL   0x26  /* 0010 0110 */
#define ESC     0x27  /* 0010 0111 */
#define SPCS    0x28  /* 0010 1000 */
#define IDENT   0x29  /* 0010 1001 */

#define OPT     0x2D  /* 0010 1101 */
#define REF     0x2E  /* 0010 1110 */
#define CLO     0x2F  /* 0010 1111 */

/* Codes between 0x30 and 0x39 ('0'-'9') must not be used! */

#define BOL     0x3A  /* 0011 1010  ':' */ 
#define EOL     0x3B  /* 0011 1011  ';' */
#define BOT     0x3C  /* 0011 1100      */
#define BOW     0x3D  /* 0011 1101  '<' */
#define EOT     0x3E  /* 0011 1110      */
#define EOW     0x3F  /* 0011 1111  '>' */

/* 01xx xxxx    matches a single char and, thus, can be included
                in a closure. */
                
/* Codes 0x41-0x48 are special! See character class in '<re_exec.c> */   
                
#define ANYUPR  0x41  /* 0100 0001 */
#define ANYLWR  0x42  /* 0100 0010 */
#define ALNUM   0x43  /* 0100 0011 */
#define DIGIT   0x44  /* 0100 0100 */
#define XDIGIT  0x45  /* 0100 0101 */
#define SPACE   0x46  /* 0100 0110 */
#define ALPHA   0x47  /* 0100 0111 */
#define PUNCT   0x48  /* 0100 1000 */

#define CTRL    0x49  /* 0100 1001 */
#define ANY     0x4A  /* 0100 1010 */
#define NL      0x4B  /* 0100 1011 */
#define WORDC   0x4C  /* 0100 1100 */
#define SPCTAB  0x4D  /* 0100 1101 */

/*   This relations must be true: 
.$
     CHI = CHR +1
     STR = CHR +2
     STI = STR +1
..
*/

#define CHR     0x51  /* 0101 0001 */
#define CHI     0x52  /* 0101 0010 */
#define STR     0x53  /* 0101 0011 */
#define STI     0x54  /* 0101 0100 */

/* Actually strings match more than a single char and, thus,
   couldn't be incuded in a closure. To fix this, the last
   character will be converted in a single char (CHR or CHI).
*/   

/* 1xxx xxxx    Character class. No codes above 0x80 can be used! */   
#define CCL     0x80  /* 1000 0000 */

#define END     0x00  /* 0000 0000 */

#define ISANYUPR  0x01
#define ISANYLWR  0x02
#define ISALNUM   0x04
#define ISDIGIT   0x08
#define ISXDIGIT  0x10
#define ISSPACE   0x20
#define ISALPHA   0x40
#define ISPUNCT   0x80

#define DEBUG

/*
 * The following defines are not meant to be changeable.
 * They are for readability only.
 */
#define MAXCHR  256
#define CHRBIT  8
#define BITBLK  MAXCHR/CHRBIT
#define BLKIND  0370
#define BITIND  07

#ifdef NO_UCHAR
typedef char CHAR;
#else
typedef unsigned char CHAR;
#endif

static CHAR bitarr[] = {1,2,4,8,16,32,64,128};


/*
 * skip values for CLO XXX to skip past the closure
 */

#define ANYSKIP  2           /* [CLO] ANY END ...       */
#define CHRSKIP  3           /* [CLO] CHR chr END ...     */

int CCLSKIP(char *p);
  
#include "re.h"
int re_isa     (unsigned char c, unsigned char what);
int re_isinset (unsigned char *x, int y);


#endif
