/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef AUDIOPROCESSOR_H
#define AUDIOPROCESSOR_H

#include "listable.h"
#include "types.h"

namespace bipscript {

class AudioProcessor : public Listable<AudioProcessor> {
    samples_t processedUntil;
    bool active;

  public:
    AudioProcessor() : processedUntil(0), active(true) {}
    /**
     *
     * Main audio engine process method
     *
     * Returns true if the object would like to stay alive
     *
     * Runs in the process thread.
     */
    bool process(samples_t time) {
        if (processedUntil >= time) {
            return active;
        }
        active = doProcess(time);
        processedUntil = time;
        return active;
    }
    /**
     * Called when a restart has been requested so objects can flush/recycle queued events.
     *
     * Runs in the process thread.
     */
    virtual void restart() = 0;
    /**
     * Called when a restart has been requested so objects can clean up
     *
     * Runs after the process restart()
     *
     * Runs in the script thread.
     */
    virtual void restartScript() {}

  protected:
    virtual bool doProcess(samples_t time) = 0;
};

} // namespace bipscript

#endif // AUDIOPROCESSOR_H
