/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BUFFER_H
#define BUFFER_H

#include "refcountable.h"
#include "types.h"

#include <algorithm>

namespace bipscript {
namespace audio {

class Buffer : public RefCountable<Buffer> {
    float *mdata;
    samples_t msize;

  public:
    Buffer(float *data, samples_t size) : mdata(data), msize(size) {}
    ~Buffer() { delete[] mdata; }
    samples_t size() { return msize; }
    float *data() { return mdata; }
    void write(float *data, samples_t start, samples_t size) {
        std::copy(data, data + size, &mdata[start]);
    }
    void regain(float ratio) {
        for (samples_t i = 0; i < msize; i++) {
            mdata[i] *= ratio;
        }
    }
};

} // namespace audio
} // namespace bipscript

#endif // BUFFER_H
