/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CLIP_H
#define CLIP_H

#include "buffertype.h"
#include "objectcache.h"

#include <sndfile.h>
#include <stdint.h>

namespace bipscript {
namespace audio {

class Buffer;

class Clip {
    audio::Buffer **mbuffer;
    uint64_t msize;
    uint32_t mchannels;
    uint32_t mframerate;

  public:
    Clip(const char *file);
    Clip(bipscript::Buffer &buffer);
    Clip(uint32_t channels, uint64_t size, uint32_t framerate);
    Clip(Clip &copy);
    ~Clip();
    void restartScript() {} // TODO: can remove?
    uint64_t size() { return msize; }
    uint32_t channels() { return mchannels; }
    Buffer *buffer(uint32_t channel);
    Buffer *ibuffer(uint32_t channel) { return mbuffer[channel]; }
    float lufs();
    void regain(float ratio);

  private:
    bool load(SNDFILE *file, SF_INFO &sfinfo);
};

class ClipFactory : public ActiveCache<Clip> {
  public:
    static ClipFactory &instance() {
        static ClipFactory instance;
        return instance;
    }
    void newObject(Clip *) override {}
    void removeObject(Clip *) override {}
    Clip *loadClip(const char *file);
    Clip *loadClip(bipscript::Buffer &bytes);
};

} // namespace audio
} // namespace bipscript

#endif // CLIP_H
