/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CLOSURE_H
#define CLOSURE_H

#include "scripttypes.h"

namespace bipscript {

class Closure : public ScriptFunction {
    SQInstance *mcontext;

  public:
    Closure(ScriptFunction &function, SQInstance *context)
        : ScriptFunction(function), mcontext(context) {
        if (context) {
            HSQOBJECT obj;
            obj._type = OT_INSTANCE;
            obj._unVal.pInstance = context;
            sq_addref(vm, &obj);
        }
    }
    void execute() {
        // push function pointer as closure
        sq_pushobject(vm, function());
        if (!mcontext) {
            // push closure root as "this" pointer
            sq_getclosureroot(vm, -1);
        } else {
            HSQOBJECT contextObj;
            contextObj._type = OT_INSTANCE;
            contextObj._unVal.pInstance = mcontext;
            sq_pushobject(vm, contextObj);
        }
        // sublclasses add optional parameters
        addParameters();
        // attempt to call function
        if (SQ_FAILED(sq_call(vm, numargs(), SQFalse, SQTrue))) {
            // TODO: error handling
        }
        // pop closure
        sq_pop(vm, 1);
    }
    virtual ~Closure() {
        if (mcontext) {
            HSQOBJECT contextObj;
            contextObj._type = OT_INSTANCE;
            contextObj._unVal.pInstance = mcontext;
            sq_release(vm, &contextObj);
        }
    }
    virtual void finished() = 0;

  protected:
    void addFloat(float f) { sq_pushfloat(vm, f); }
    void addInteger(int i) { sq_pushinteger(vm, i); }
    void addString(const char *s, int size) { sq_pushstring(vm, s, size); }
    void addObject(HSQOBJECT &obj) { sq_pushobject(vm, obj); }
    virtual void addParameters() = 0;
};

} // namespace bipscript

#endif // CLOSURE_H
