/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef COLLECTOR_H
#define COLLECTOR_H

#include "listable.h"
#include "threadqueue.h"

namespace bipscript {

class GarbageCollector : public Listable<GarbageCollector> {
  public:
    virtual void free() = 0;
};

template <class T> class Collector : public GarbageCollector {
    ThreadQueue<T> mqueue;

  public:
    Collector() : mqueue(4096) {}
    void recycle(T *obj) { mqueue.add(obj); }
    void free() override {
        T *obj;
        while (mqueue.pop(obj)) {
            delete obj;
        }
    }
};

template <class T> class RefCollector : public GarbageCollector {
    ThreadQueue<T> mqueue;

  public:
    RefCollector() : mqueue(4096) {}
    void recycle(T *obj) { mqueue.add(obj); }
    void free() override {
        T *obj;
        while (mqueue.pop(obj)) {
            int delref = obj->delref();
            if (delref == 0) {
                delete obj;
            }
        }
    }
};

} // namespace bipscript

#endif // COLLECTOR_H
