/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef COLLECTORTHREAD_H
#define COLLECTORTHREAD_H

#include <pthread.h>

#include "collector.h"
#include "queuelist.h"

namespace bipscript {

class CollectorThread {
    pthread_t thread;
    QueueList<GarbageCollector> queueList;
    std::atomic<bool> finished;
    CollectorThread();

  public:
    static CollectorThread &instance() {
        static CollectorThread instance;
        return instance;
    }
    void addCollector(GarbageCollector *collector) { queueList.add(collector); }
    void removeCollector(GarbageCollector *collector) { queueList.remove(collector); }
    void run();
    void shutdown() {
        finished.store(true);
        pthread_join(thread, nullptr);
    }
};

} // namespace bipscript

#endif // COLLECTORTHREAD_H
