/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef EXTENSION_H
#define EXTENSION_H

#include <string>
#include <vector>

#include "bipxtapi.h"
#include "squirrel.h"

void bipxt_setvm(HSQUIRRELVM);

namespace bipscript {

#define BIP_XT_PATH "bipscript/xt/"

class Extension {
    static const int BIPXT_VERSION = 1;
    void *handle;

  public:
    Extension(const std::string &);
    ~Extension();
    void import(HSQUIRRELVM vm, const char *name);
    void shutdown();
};

class ExtensionManager {
    std::vector<Extension *> extensions;
    std::vector<BXTMethod> ehandlers;
    std::vector<std::string> mreserved;
    std::vector<std::string> mimported;
    std::vector<std::string> mextpath;
    ExtensionManager() {
        std::string names[] = {"Audio", "IO",  "Jack", "Lv2",    "Math",
                               "Midi",  "Net", "Osc",  "Script", "Time"};
        for (int i = 0; i < 2; i++) {
            mreserved.push_back(names[i]);
        }
        // TODO: what about systems with nonstandard locations
        mextpath.push_back("/usr/lib/" BIP_XT_PATH);
        mextpath.push_back("/usr/local/lib/" BIP_XT_PATH);
    }

  public:
    static ExtensionManager &instance() {
        static ExtensionManager instance;
        return instance;
    }
    void import(HSQUIRRELVM vm, const char *name);
    void addHandler(BXTMethod handler) { ehandlers.push_back(handler); }
    void shutdown();
    bool eventLoop();
};

} // namespace bipscript

#endif // EXTENSION_H
