/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef HEAPBUFFER_H
#define HEAPBUFFER_H

#include <stddef.h>

namespace bipscript {

class HeapBuffer {
    unsigned char *mptr;
    size_t msize;

  public:
    HeapBuffer() { alloc(256); }
    ~HeapBuffer() { delete[] mptr; }
    unsigned char *ptr() { return mptr; }
    void resize(size_t size) {
        if (size > msize) {
            delete[] mptr;
            alloc(size);
        }
    }
    unsigned char &operator[](int index) { return mptr[index]; }

  private:
    void alloc(size_t size) {
        mptr = new unsigned char[size];
        msize = size;
    }
};

} // namespace bipscript

#endif // HEAPBUFFER_H
