/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ZIPFILE_H
#define ZIPFILE_H

#include "buffer.h"
#include "heapbuffer.h"
#include "io.h"
#include "miniz/miniz.h"
#include "refcountable.h"

namespace bipscript {
namespace io {

class Zip : public RefCountable<Zip> {
    mz_zip_archive zip;
    uint32_t mentries;
    HeapBuffer textbuffer;

  public:
    Zip(const char *fpath);
    ~Zip() { mz_zip_reader_end(&zip); }
    uint32_t len() { return entries(); } // for iterator
    uint32_t entries() { return mentries; }
    Path *entry(const char *path);
    Buffer *bytes(const std::string &path);
    const char *text(const std::string &path);
    Path *operator[](int index);

  private:
    mz_zip_archive_file_stat statFile(int index);
    mz_zip_archive_file_stat find(const char *path);
};

class ZipFolder : public Path {
  public:
    ZipFolder(const char *path) : Path(path) {}
    bool exists() override { return true; }
    bool folder() override { return true; }
};

class ZipFile : public Path {
    Zip *mparent;
    uint64_t msize;

  public:
    ZipFile(const char *path, Zip *parent, uint64_t len)
        : Path(path), mparent(parent->addref()), msize(len) {}
    ~ZipFile();
    bool exists() override { return true; }
    bool folder() override { return false; }
    uint64_t size() { return msize; }
    Buffer *bytes() { return mparent->bytes(mpath); }
    const char *text() { return mparent->text(mpath); }
};

} // namespace io
} // namespace bipscript

#endif // ZIPFILE_H
