/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "jackclock.h"
#include "audioengine.h"

namespace bipscript {
namespace jack {

void Clock::update(samples_t) {
    jack_position_t pos;
    bool rolling(transport.query(&pos) == JackTransportRolling);
    // is this a reposition?
    current.reposition(pos.frame != current.frame() + current.size());
    // set frames
    current.frame(pos.frame);
    current.size(period());
    if (pos.valid & JackPositionBBT) {
        if (current.measure() == 0.0f) { // last position wasn't valid
            current.reposition(true);
        }
        // set time signature and bpm
        float tempo = static_cast<float>(pos.beats_per_minute);
        current.bpm(tempo);
        mtempo.store(tempo);
        current.signature().numerator(static_cast<uint16_t>(pos.beats_per_bar));
        current.signature().denominator(static_cast<uint16_t>(pos.beat_type));
        this->ticksPerBeat = static_cast<uint16_t>(pos.ticks_per_beat);
        double measure = pos.bar + (pos.beat - 1 + (pos.tick / pos.ticks_per_beat)) *
                                       (current.signature().denominator() / 4) /
                                       current.signature().numerator();
        current.measure(static_cast<float>(measure));
        this->mtimer.running(rolling);
    } else {
        // invalid external BBT
        current.measure(0.0f);
        this->mtimer.running(false);
    }
    updateRemotes();
    //    printf("-- TM%d %d -> %d/%d/%d [%f/%f]\n", pos.valid, pos.frame, pos.bar, pos.beat,
    //    pos.tick, pos.beats_per_bar, pos.beat_type); printf("ex: "); current.print();
}

// TODO: should cache?
Clock *ClockFactory::newClock() { return AudioEngine::instance().externalTimeClock(); }

} // namespace jack
} // namespace bipscript
