/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef JACKTRANSPORT_H
#define JACKTRANSPORT_H

#include <jack/jack.h>

namespace bipscript {
namespace jack {

class Transport {
    jack_client_t *client;

  public:
    Transport(jack_client_t *client) : client(client) {}
    void stop() { jack_transport_stop(client); }
    void start() { jack_transport_start(client); }
    void relocate(jack_nframes_t frame) { jack_transport_locate(client, frame); }
    jack_transport_state_t query(jack_position_t *pos) { return jack_transport_query(client, pos); }
};

class Position {
    jack_position_t pos;
    float mmeasure;

  public:
    Position(jack_position_t &pos) : pos(pos) {
        mmeasure = pos.valid & JackPositionBBT
                       ? pos.bar + (pos.beat - 1 + (pos.tick / pos.ticks_per_beat)) *
                                       (pos.beat_type / 4) / pos.beats_per_bar
                       : 0.0;
    }
    uint32_t bar() { return pos.bar; }
    uint32_t beat() { return pos.beat; }
    uint32_t tick() { return pos.tick; }
    float measure() { return mmeasure; }
};

} // namespace jack
} // namespace bipscript

#endif // JACKTRANSPORT_H
