/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef MEMORY_H
#define MEMORY_H

#include <fstream>
#include <sys/resource.h>
#include <unistd.h>

namespace bipscript {
namespace system {

class Memory {
    long mtotal;
    long mfree;
    long mused;
    long mresident;

  public:
    Memory() {
        // total and free via sysconf
        long pagesizekb = sysconf(_SC_PAGE_SIZE) / 1024;
        mtotal = sysconf(_SC_PHYS_PAGES) * pagesizekb;
        mfree = sysconf(_SC_AVPHYS_PAGES) * pagesizekb;

        // used and resident via /proc
        int stotal = 0, sresident = 0, shared = 0;
        std::ifstream buffer("/proc/self/statm");
        buffer >> stotal >> sresident >> shared;
        buffer.close();
        mused = stotal * pagesizekb;
        mresident = sresident * pagesizekb;
    }
    long total() { return mtotal; }
    long free() { return mfree; }
    long used() { return mused; }
    long resident() { return mresident; }
};

} // namespace system
} // namespace bipscript

#endif // MEMORY_H
