/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef METHODQUEUE_H
#define METHODQUEUE_H

#include <boost/lockfree/queue.hpp>
#include <boost/next_prior.hpp>

#include "closure.h"

namespace bipscript {

/**
 * A global queue that passes methods from process thread to script thread for
 * immediate execution (e.g. callbacks)
 *
 */
class MethodQueue {
    boost::lockfree::queue<Closure *> dispatchQueue; // any thread -> script thread
    std::atomic<uint> mactive;
    MethodQueue();

  public:
    static MethodQueue &instance() {
        static MethodQueue instance;
        return instance;
    }
    void dispatch(Closure *function);
    Closure *next();
    void finished(Closure *closure);
    void scriptReset();
    bool active() { return mactive.load() > 0; }
};

} // namespace bipscript

#endif // METHODQUEUE_H
