/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef MIDISEQUENCE_H
#define MIDISEQUENCE_H

#include <algorithm>
#include <vector>

#include "midievent.h"
#include "scripttypes.h"

namespace bipscript {
namespace midi {

class SequenceFilter : public ScriptFunction {
  public:
    SequenceFilter(ScriptFunction &function);
    bool execute(size_t index, MessageEvent *event);
};

class Note;
class Pattern;

class Sequence {
    std::vector<MessageEvent *> eventlist;

  public:
    Sequence() {}
    Sequence(Sequence &copy);
    ~Sequence();
    uint32_t len() { return eventlist.size(); }
    float length() { // TODO: figure out names before exposing: len -> size?
        return eventlist.size() ? eventlist[eventlist.size() - 1]->measure() - 1.0 : 0;
    }
    MessageEvent *add(Message &mesg, float measure);
    void add(Note &note, float measure);
    void add(Pattern &pattern, float measure = 1.0);
    void add(Sequence &sequence, float measure = 1.0);
    void add(MessageEvent *event);
    void remove(MessageEvent *evt) {
        eventlist.erase(std::remove(eventlist.begin(), eventlist.end(), evt), eventlist.end());
    }
    void clear();
    Sequence *filter(ScriptFunction func);
    MessageEvent *operator[](int index) {
        if (index < 0 || index >= static_cast<int>(eventlist.size())) {
            return nullptr;
        }
        return eventlist[index];
    }
    Pattern *pattern();
};

} // namespace midi
} // namespace bipscript

#endif // MIDISEQUENCE_H
