/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef OSCOUTPUT_H
#define OSCOUTPUT_H

#include "objectcache.h"
#include "oscmessage.h"
#include "processorthread.h"
#include "remotetimeset.h"
#include "sendqueue.h"

#include <lo/lo.h>

namespace bipscript {
namespace osc {

class MessageEvent : public Event<Message> {
  public:
    MessageEvent(Message *mesg, Command::Type type, const float measure, time::Clock &clock)
        : Event<Message>(mesg, type, measure, clock) {}
    Message *message() { return &Event<Message>::message(); }
};

class Output : public ProcessorThread {
    lo_address loAddress;
    RemoteTimelineSet<Message> timelineset;
    SendQueue<Message> sendqueue;

  public:
    Output(const char *host, int port);
    ~Output() { lo_address_free(loAddress); }
    void remove(MessageEvent &);
    MessageEvent *schedule(Message &message, float measure, Clock &clock = Clock::defaultClock());
    void send(Message &message) { sendqueue.add(new Message(message)); }
    // cache managed object
    void restartScript() { timelineset.restartScript(); }

  protected:
    bool process() override;
    bool restarted() override { return timelineset.restart(); }

  private:
    void sendMessage(Message &message);
};

class OutputFactory : public ActiveCache<Output> {
    void newObject(Output *output) override { ScriptHost::instance().addProcessorThread(output); }
    void removeObject(Output *output) override {
        output->join();
        ScriptHost::instance().removeProcessorThread(output);
        delete output; // TODO: script thread?
    }

  public:
    static OutputFactory &instance() {
        static OutputFactory instance;
        return instance;
    }
    ~OutputFactory() { removeAll(); }
    Output *newOscOutput(const char *host, int port);
};

} // namespace osc
} // namespace bipscript

#endif // OSCOUTPUT_H
