/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <chrono>
#include <thread>

#include "processorthread.h"

namespace bipscript {

void *run_output(void *arg) {
    ((ProcessorThread *)arg)->run();
    return 0;
}

void ProcessorThread::start() {
    int result = pthread_create(&thread, 0, run_output, this);
    if (result) {
        throw std::logic_error("could not create processor thread");
    }
}

void ProcessorThread::join() {
    mfinished.store(true);
    pthread_join(thread, nullptr);
}

void ProcessorThread::run() {
    while (!mfinished.load()) {
        bool scriptRunning = ScriptHost::instance().running();
        bool pactive = process();
        // restart requested?
        if (mrestarting.load()) {
            while (!restarted()) {
                std::this_thread::sleep_for(std::chrono::milliseconds(1));
            }
            mrestarting.store(false);
            if (!mresetproof) {
                mfinished.store(true);
                mactive.store(false);
                return;
            }
        }
        // store active
        mactive.store(scriptRunning || pactive);
        std::this_thread::sleep_for(std::chrono::milliseconds(mperiod));
    }
}

} // namespace bipscript
