/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PROCESSORTHREAD_H
#define PROCESSORTHREAD_H

#include <atomic>
#include <pthread.h>

#include "processorengine.h"
#include "scripthost.h"

namespace bipscript {

void *run_output(void *arg);

/*
 * pthread implementation
 */
class ProcessorThread : public ProcessorEngine {

    pthread_t thread;
    bool mresetproof;              // end thread on script reset?
    std::atomic<bool> mrestarting; // script thread requests restart
    std::atomic<bool> mactive;     // timeline holds active events
    std::atomic<bool> mfinished;   // script thread cache requests deletion
    std::atomic<int> mperiod;      // time to sleep between runs in ms

  public:
    ProcessorThread(bool resetproof, uint32_t period)
        : mresetproof(resetproof), mrestarting(false), mactive(true), mfinished(false),
          mperiod(period) {}

    void start();
    void join();
    void period(int period) { mperiod.store(period); }
    // ProcessorThreadEngine API, called from script thread
    void restart() override { mrestarting.store(true); }
    bool restarting() override { return mrestarting.load(); }
    bool active() override { return mactive; }
    // runs in new pthread
    void run();

  protected:
    virtual bool process() = 0;
    virtual bool restarted() = 0;
};

} // namespace bipscript

#endif // PROCESSORTHREAD_H
