/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef STDPIPES_H
#define STDPIPES_H

#include "bindutil.h"
#include "fileio.h"

#include "virtualmachine.h"

namespace bipscript {

class StdPipe {
  public:
    static void bindObject(HSQUIRRELVM vm, const char *name, HSQOBJECT objtype, void *instance) {
        sq_pushstring(vm, name, -1);
        sq_pushobject(vm, objtype);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, instance);
        sq_newslot(vm, -3, true);
    }
};

class StdIn : public FileHandleReader {
  public:
    static StdIn &instance() {
        static StdIn instance;
        return instance;
    }
    StdIn() : FileHandleReader(stdin) {}
    static void bindObject(HSQUIRRELVM vm) {
        StdPipe::bindObject(vm, "stdin", binding::readerObject, instance().addref());
    }
};

class StdOut : public FileHandleWriter {
  public:
    static StdOut &instance() {
        static StdOut instance;
        return instance;
    }
    StdOut() : FileHandleWriter(stdout) {}
    static void bindObject(HSQUIRRELVM vm) {
        StdPipe::bindObject(vm, "stdout", binding::writerObject, &instance());
    }
};

class StdErr : public FileHandleWriter {
  public:
    static StdErr &instance() {
        static StdErr instance;
        return instance;
    }
    StdErr() : FileHandleWriter(stderr) {}
    static void bindObject(HSQUIRRELVM vm) {
        StdPipe::bindObject(vm, "stderr", binding::writerObject, &instance());
    }
};

} // namespace bipscript

#endif // STDPIPES_H
