/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef THREADQUEUE_H
#define THREADQUEUE_H

#include <boost/lockfree/spsc_queue.hpp>
#include <boost/next_prior.hpp>

#include <chrono>
#include <thread>

namespace bipscript {

/**
 * Abstraction for a single producer, single consumer lockless queue
 */
template <class T> class ThreadQueue {
    boost::lockfree::spsc_queue<T *> queue;

  public:
    ThreadQueue(size_t size) : queue(size) {}
    void add(T *elem) {
        while (!queue.push(elem)) {
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
    }
    bool pop(T *&elem) { return queue.pop(elem); }
};

} // namespace bipscript

#endif // THREADQUEUE_H
