/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BASICCLOCK_H
#define BASICCLOCK_H

#include "audioengine.h"
#include "clock.h"
#include "clockcollector.h"
#include "statusserver.h"

namespace bipscript {
namespace time {

class BasicClock : public Clock {
    // script state
    bool mrunning;
    Signature msignature;
    void running(bool value) {
        mrunning = value;
        commands.add(new StateChange(value));
    }

  protected:
    Window following;
    void clockChanges();

  public:
    BasicClock(float bpm) : Clock(bpm), mrunning(false) { following.bpm(bpm); }
    BasicClock(float bpm, Signature &signature) : Clock(bpm), mrunning(false) {
        msignature = signature;
        following.bpm(bpm);
        following.signature(signature);
    }
    Signature *signature() { return &msignature; }
    void signature(Signature *signature);
    void start() override { running(true); }
    void stop() override { running(false); }
    void restart() {
        commands.add(new Restart());
        mrunning = true;
    }
    void update(samples_t) override;
    bool hasHistory() override { return false; } // TODO: is this true?
    void recycle() override {
        StatusServer &statusserver = StatusServer::instance();
        if (statusserver.active()) {
            statusserver.finishClock(this);
        }
        ClockCollector::instance().recycle(this);
    } // TODO: move to base
};

class BasicClockFactory {
  public:
    static BasicClockFactory &instance() {
        static BasicClockFactory instance;
        return instance;
    }
    BasicClock *basicClock(float bpm) {
        AudioEngine &engine = AudioEngine::instance();
        BasicClock *clock = new BasicClock(bpm);
        engine.addClock(clock);
        StatusServer &statusserver = StatusServer::instance();
        if (statusserver.active()) {
            statusserver.newClock(clock, "basic", bpm);
        }
        return clock;
    }
    BasicClock *basicClock(float bpm, Signature &sig) {
        AudioEngine &engine = AudioEngine::instance();
        BasicClock *clock = new BasicClock(bpm, sig);
        engine.addClock(clock);
        StatusServer &statusserver = StatusServer::instance();
        if (statusserver.active()) {
            statusserver.newClock(clock, "basic", bpm, sig.numerator(), sig.denominator());
        }
        return clock;
    }
};

} // namespace time
} // namespace bipscript

#endif // BASICCLOCK_H
