﻿/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "clock.h"
#include "clockcollector.h"

#include "audioengine.h"
#include "basicclock.h"

#include <cmath>

namespace bipscript {
namespace time {

samples_t Window::msampleRate;
Clock *Clock::defclock;
samples_t Clock::mperiod;
Collector<RemoteWindow> RemoteWindow::collector;

Clock &Clock::defaultClock() {
    if (!defclock) {
        // TODO define default BPM somewhere
        defclock = BasicClockFactory::instance().basicClock(120);
    }
    return *defclock;
}

void Clock::defaultClock(Clock *clock) { defclock = clock; }

void Clock::running(bool value) {
    RemoteTimer *timer = remoteTimers.first();
    while (timer) {
        timer->running(value);
        timer = remoteTimers.next(timer);
    }
    mtimer.running(value);
}

void Clock::updateRemotes() {
    RemoteTimer *timer = remoteTimers.first();
    while (timer) {
        timer->update();
        timer = remoteTimers.next(timer);
    }
}

void Clock::finished() {
    // finish remote timers
    RemoteTimer *timer = remoteTimers.first();
    while (timer) {
        timer->finish();
        timer = remoteTimers.next(timer);
    }
}

float Timer::offsetToMeasure(samples_t offset) {
    Window *window = firstpos();
    while (window) {
        if (offset < window->size()) {
            return window->measure() + window->moffset(offset);
        }
        offset -= window->size();
        window = nextpos(window);
    }
    return 0.0; // not rolling
}

bool RemoteTimer::finished() {
    if (mfinished.load()) {
        // recycle remaining remote windows
        if (last) {
            RemoteWindow::recycle(last);
            last = nullptr;
        }
        RemoteWindow *next;
        while (queue.pop(next)) {
            RemoteWindow::recycle(next);
        }
        // recycle this timer
        ClockCollector::instance().recycle(this);
        return true;
    }
    return false;
}

void RemoteTimer::update() {
    Window *window = original.firstpos();
    while (!mfinished.load() && window) {
        queue.add(new RemoteWindow(*window));
        window = original.nextpos(window);
    }
}

Window *RemoteTimer::firstpos() {
    RemoteWindow *first;
    if (last) {
        RemoteWindow::recycle(last);
    }
    last = queue.pop(first) ? first : 0;
    return last;
}

Window *RemoteTimer::nextpos(Window *) {
    RemoteWindow *next;
    RemoteWindow::recycle(last);
    last = queue.pop(next) ? next : 0;
    return last;
}

} // namespace time
} // namespace bipscript
