/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef LINK_H
#define LINK_H

#include "audioengine.h"
#include "clock.h"
#include "clockchangeevent.h"
#include "statusserver.h"

#include "ableton/Link.hpp"
#include "ableton/link/HostTimeFilter.hpp"

namespace bipscript {
namespace time {

typedef ableton::link::HostTimeFilter<ableton::link::platform::Clock> HostTimeFilter;

class Link : public Clock {
    ableton::Link mlink;
    HostTimeFilter mHostTimeFilter;
    SortedQueueList<ClockChangeEvent> eventBuffer;
    ClockChangeEvent *nextEvent;
    Window startWindow; // extra time window to straddle frame zero
    float initbpm;
    samples_t mframe;
    int mquantum;

  public:
    Link();
    Link(float bpm, int quantum);
    Link(float bpm, Signature &signature);
    Link(float bpm) : Link(bpm, 4) {}
    ~Link();
    bool hasHistory() override { return false; }
    void start() override;
    void stop() override;
    void schedule(float tempo, float measure) {
        validateTempo(tempo);
        eventBuffer.addEvent(ClockChangeEvent::tempoChangeEvent(tempo, measure));
    }
    void update(samples_t) override;
    void recycle() override;
};

class LinkFactory {
    Link *mlink;

  public:
    static LinkFactory &instance() {
        static LinkFactory instance;
        return instance;
    }
    LinkFactory() : mlink(nullptr) {}
    ~LinkFactory() { delete mlink; }
    Link *newLink();
    Link *newLink(float bpm);
    Link *newLink(float bpm, Signature &sig);
};

} // namespace time
} // namespace bipscript

#endif // LINK_H
