/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef XTFUNCTIONCALL_H
#define XTFUNCTIONCALL_H

#include <list>

namespace bipscript {

enum class XtCallArgType { ANY, INTEGER, STRING, INSTANCE };

struct XtCallArg {
    XtCallArgType type;
    union {
        int ivalue;
        void *ovalue;
        const char *svalue;
    };
    XtCallArg(int value) : type(XtCallArgType::INTEGER), ivalue(value) {}
    XtCallArg(const char *value) : type(XtCallArgType::STRING), svalue(value) {}
    XtCallArg(void *value) : type(XtCallArgType::INSTANCE), ovalue(value) {}
    XtCallArg(void *value, XtCallArgType type) : type(type), ovalue(value) {}
};

class XtFunctionCall {
    void *mfunction;
    std::list<XtCallArg> arglist;

  public:
    XtFunctionCall(void *function) : mfunction(function) {}
    void addInt(int value) { arglist.push_back(XtCallArg(value)); }
    void addString(const char *obj) { arglist.push_back(XtCallArg(obj)); }
    void addObject(void *obj) { arglist.push_back(XtCallArg(obj)); }
    void addValue(void *value) { arglist.push_back(XtCallArg(value, XtCallArgType::ANY)); }
    void *function() { return mfunction; }
    int nargs() { return arglist.size(); }
    std::list<XtCallArg> &args() { return arglist; }
};

} // namespace bipscript
#endif // XTFUNCTIONCALL_H
